@@ -1,7 +1,7 @@
 module Email ( make_email, read_email ) where
 
 import Char ( ord, digitToInt, isHexDigit )
-import Printer ( Doc, ($$), (<+>), (<>), text, packedString, renderPS )
+import Printer ( Doc, ($$), (<+>), (<>), text, empty, packedString, renderPS)
 import FastPackedString ( PackedString, tailPS, nullPS, dropWhitePS,
                           indexPSW, packString, headPS, indexPS,
                           lengthPS, generatePS, dropPS, linesPS,
@@ -89,20 +89,26 @@ decode (ps:pss) buf bufi
           toWord8 :: Int -> Word8
           toWord8 = fromIntegral
 
-make_email :: String -> Doc -> Doc -> String -> Doc
-make_email repodir contents bundle filename =
+make_email :: String -> (Maybe Doc) -> Doc -> (Maybe String) -> Doc
+make_email repodir mcontents bundle mfilename =
     text "DarcsURL:" <+> text repodir
  $$ text "MIME-Version: 1.0"
  $$ text "Content-Type: multipart/mixed; boundary=\"=_\""
  $$ text ""
  $$ text "--=_"
- $$ text "Content-Type: text/plain"
- $$ text "Content-Transfer-Encoding: quoted-printable"
- $$ text ""
- $$ packedString (qpencode (renderPS contents))
- $$ text ""
- $$ text "--=_"
- $$ text "Content-Type: text/x-darcs-patch; name=\"" <> text filename <> text "\""
+ $$ (case mcontents of
+       Just contents ->
+            text "Content-Type: text/plain"
+         $$ text "Content-Transfer-Encoding: quoted-printable"
+         $$ text ""
+         $$ packedString (qpencode (renderPS contents))
+         $$ text ""
+         $$ text "--=_"
+       Nothing -> empty)
+ $$ text "Content-Type: text/x-darcs-patch" <>
+      (case mfilename of
+         Just filename -> text "; name=\"" <> text filename <> text "\""
+         Nothing -> empty)
  $$ text "Content-Transfer-Encoding: quoted-printable"
  $$ text "Content-Description: A darcs patch for your repository!"
  $$ text ""
