@@ -1,4 +1,18 @@
 {-# OPTIONS -fffi #-}
+#ifdef HAVE_CURSES
+#ifdef HAVE_TERMIO_H
+{-# INCLUDE <termio.h> #-}
+#endif
+{-# INCLUDE <curses.h> #-}
+{-# INCLUDE <term.h> #-}
+#endif
+-- The termio bit above is a grim hack for Solaris 10 (at least).
+-- Without it, SGTTY isn't declared.  Something in the ghc C headers
+-- which get included before term.h in the generated C is clobbering
+-- it.  (If you edit that C and put the curses.h and term.h first, the
+-- error goes away.)
+-- [Putting this comment before any of the INCLUDE lines prevents them
+-- being acted on :-(.]
 module External (
     copyFileOrUrl, copyFilesOrUrls,
     cloneTree, cloneTreeExcept, clonePartialsTree, clonePaths,
@@ -31,7 +45,7 @@ import Autoconf ( use_color )
 #endif
 import Workaround ( createLink, createDirectoryIfMissing )
 
-import AtExit ( atexit )
+import Global ( atexit, sshControlMasterDisabled )
 import DarcsFlags ( DarcsFlag( Quiet, SignAs, Sign, SignSSL,
                                Verbose, Verify, VerifySSL ) )
 import DarcsUtils ( withCurrentDirectory )
@@ -41,14 +55,14 @@ import FastPackedString ( PackedString, readFilePS, gzReadFilePS, writeFilePS,
                           nullPS, nilPS, concatPS
                         )
 import Lock ( withTemp, withOpenTemp, readDocBinFile,
-              canonFilename, writeBinFile, writeDocBinFile,
+              canonFilename, writeDocBinFile,
               tempdir_loc,
             )
 import CommandLine ( parseCmd, addUrlencoded )
 import Autoconf ( have_libcurl, have_sendmail, have_mapi, sendmail_path, darcs_version )
 import Curl ( copyUrl )
 import Curl ( Cachable(..) )
-import Exec ( exec )
+import Exec ( exec, Redirects, Redirect(..), )
 import DarcsURL ( is_file, is_url, is_ssh )
 import DarcsUtils ( catchall )
 import Printer ( Doc, hPutDoc, hPutDocLn, ($$), (<+>), renderPS,
@@ -153,7 +167,7 @@ copyRemote u v cache =
          Just get ->
            do let cmd = head $ words get
                   args = tail $ words get
-              r <- exec cmd (args++[u]) "/dev/null" v
+              r <- exec cmd (args++[u]) (Null, File v, AsIs)
               when (r /= ExitSuccess) $
                   fail $ "(" ++ get ++ ") failed to fetch: " ++ u
 
@@ -164,9 +178,7 @@ copyRemoteNormal u v cache = if have_libcurl
 
 copySSH :: String -> FilePath -> IO ()
 copySSH uRaw f = let u = escape_dollar uRaw in do
-                 (scp, scp_args) <- getSSH SCP u
-                 r <- exec scp (scp_args++[u,f])
-                      "/dev/null" "/dev/null"
+                 r <- runSSH SCP u [] [u,f] (AsIs,AsIs,Null)
                  when (r /= ExitSuccess) $
                       fail $ "(scp) failed to fetch: " ++ u
     where {- '$' in filenames is troublesome for scp, for some reason.. -}
@@ -209,7 +221,7 @@ mgetRemotes mget u ns d = do
         args = tail $ words mget
         urls = map (\n -> u++"/"++n) nsnow
     withCurrentDirectory d $ do
-        r <- exec cmd (args++urls) "/dev/null" "/dev/null"
+        r <- exec cmd (args++urls) (Null,Null,AsIs)
         when (r /= ExitSuccess) $
             fail $ unlines $
                 ["(" ++ mget ++ ") failed to fetch files.",
@@ -241,7 +253,7 @@ wgetRemotes u ns d = do wget_command <- getEnv "DARCS_WGET" `catch`
                             do hPutStr th input
                                hClose th
                                r <- exec wget (wget_args++["-i",tn])
-                                    "/dev/null" "/dev/null"
+                                         (Null,Null,AsIs)
                                when (r /= ExitSuccess) $
                                     fail $ unlines $
                                              ["(wget) failed to fetch files.",
@@ -249,42 +261,41 @@ wgetRemotes u ns d = do wget_command <- getEnv "DARCS_WGET" `catch`
                                               "source files:"] ++ ns
 
 copySSHs :: String -> [String] -> FilePath -> IO ()
-copySSHs u ns d = do (sftp, sftp_args) <- getSSH SFTP u
-                     let path = drop 1 $ dropWhile (/= ':') u
-                         host = (takeWhile (/= ':') u)++":"
-                         cd = "cd "++path++"\n"
-                         input = cd++(unlines $ map ("get "++) ns)
-                     withCurrentDirectory d $ withOpenTemp $ \(th,tn) ->
-                         withTemp $ \sftpoutput ->
-                         do hPutStr th input
-                            hClose th
-                            r <- exec sftp (sftp_args++["-b",tn,host])
-                                 "/dev/null" sftpoutput
-                            let files = if length ns > 5
-                                          then (take 5 ns) ++ ["and "
-                                               ++ (show (length ns - 5)) ++ " more"]
-                                          else ns
-                                hint = if take 1 path == "~"
-                                         then ["sftp doesn't expand ~, use path/ instead of ~/path/"]
-                                         else []
-                            when (r /= ExitSuccess) $ do
-                                 outputPS <- readFilePS sftpoutput
-                                 fail $ unlines $
-                                          ["(sftp) failed to fetch files.",
-                                           "source directory: " ++ path,
-                                           "source files:"] ++ files ++
-                                          ["sftp output:",unpackPS outputPS] ++
-                                          hint
+copySSHs u ns d =
+  do let path = drop 1 $ dropWhile (/= ':') u
+         host = takeWhile (/= ':') u
+         cd = "cd "++path++"\n"
+         input = cd++(unlines $ map ("get "++) ns)
+     withCurrentDirectory d $ withOpenTemp $ \(th,tn) ->
+         withTemp $ \sftpoutput ->
+         do hPutStr th input
+            hClose th
+            r <- runSSH SFTP u [] [host] (File tn, File sftpoutput, Null)
+            let files = if length ns > 5
+                          then (take 5 ns) ++ ["and "
+                               ++ (show (length ns - 5)) ++ " more"]
+                          else ns
+                hint = if take 1 path == "~"
+                         then ["sftp doesn't expand ~, use path/ instead of ~/path/"]
+                         else []
+            when (r /= ExitSuccess) $ do
+                 outputPS <- readFilePS sftpoutput
+                 fail $ unlines $
+                          ["(sftp) failed to fetch files.",
+                           "source directory: " ++ path,
+                           "source files:"] ++ files ++
+                          ["sftp output:",unpackPS outputPS] ++
+                          hint
 
 
 copyRemoteCmd :: String -> FilePath -> IO ()
 copyRemoteCmd s tmp = do
     let cmd = get_ext_cmd
-    r <- stupidexec (cmd tmp s) "/dev/null" "/dev/null"
+    r <- stupidexec (cmd tmp s) (Null,Null,AsIs)
     when (r /= ExitSuccess) $
          fail $ "failed to fetch: " ++ s ++" " ++ show r
-    where stupidexec (c:args) inf outf = exec c args inf outf
-          stupidexec [] _ _ = bug "stupidexec without a command"
+    where stupidexec [] = bug "stupidexec without a command"
+          stupidexec xs = exec (head xs) (tail xs)
 
 doWithPatches :: [DarcsFlag] -> (String -> IO ()) -> [String] -> IO ()
 doWithPatches opts f patches =
@@ -305,7 +316,7 @@ doWithPatches opts f patches =
               doWithCount [] index = do
                   putStr $ if verbose then ""
                            else "\rCopying patch "++(show $ index - 1)++
-                                " of "++total++"... done!\n"
+                                " of "++total++"... done.\n"
                   hFlush stdout
 
 {-# NOINLINE get_ext_cmd #-}
@@ -338,8 +349,7 @@ get_ext_cmd' = try_cmd cmds
 --   reading its output.  Return its ExitCode
 execSSH :: String -> String -> IO ExitCode
 execSSH remoteAddr command =
-    do (ssh, ssh_args) <- getSSH SSH remoteAddr
-       exec ssh (remoteAddr:ssh_args++[command]) "/dev/null" "/dev/null"
+  runSSH SSH remoteAddr [remoteAddr] [command] (AsIs,AsIs,Null)
 
 pipeDoc_SSH_IgnoreError :: String -> [String] -> Doc -> IO Doc
 pipeDoc_SSH_IgnoreError remoteAddr args input =
@@ -347,47 +357,17 @@ pipeDoc_SSH_IgnoreError remoteAddr args input =
        execPipeIgnoreError ssh (ssh_args++ (remoteAddr:args)) input
 
 sendEmail :: String -> String -> String -> String -> String -> String -> IO ()
-sendEmail _ "" _ "" _ _ = return ()
-sendEmail f "" s cc scmd body = sendEmail f cc s "" scmd body
 sendEmail f t s cc scmd body =
-  if have_sendmail || scmd /= "" then do
-    withOpenTemp $ \(h,fn) -> do
-     hPutStr h $
-        "To: "      ++ t ++ "\n" ++
-        "From: "    ++ f ++ "\n" ++
-        "Subject: " ++ s ++ "\n" ++
-        formated_cc ++
-        "X-Mail-Originator: Darcs Version Control System\n" ++
-        "X-Darcs-Version: " ++ darcs_version ++ "\n" ++
-        body
-     hClose h
-     let ftable = [('t',t),('c',cc),('t',fn),('f',f),('s',s),('b',body)]
-     r <- execSendmail ftable scmd fn
-     when (r /= ExitSuccess) $ fail ("failed to send mail to: " ++ t)
-  else if have_mapi then do
-     r <- withCString t $ \tp ->
-           withCString f $ \fp ->
-            withCString cc $ \ccp ->
-             withCString s $ \sp ->
-              withOpenTemp $ \(h,fn) -> do
-               hPutStr h body
-               hClose h
-               writeBinFile "mailed_patch" body
-               cfn <- canonFilename fn
-               withCString cfn $ \pcfn ->
-                c_send_email fp tp ccp sp nullPtr pcfn
-     when (r /= 0) $ fail ("failed to send mail to: " ++ t)
-  else fail $ "no mail facility (sendmail or mapi) located at configure time!"
-  where formated_cc = if cc == ""
-                      then ""
-                      else "Cc: "++cc++"\n"
+  sendEmailDoc f t s cc scmd Nothing (text body)
 
+-- | Send an email, optionally containing a patch bundle
+--   (more precisely, its description and the bundle itself)
 sendEmailDoc :: String -> String -> String -> String -> String
-             -> Doc -> Doc -> Doc -> IO ()
-sendEmailDoc _ "" _ "" _ _ _ _ = return ()
-sendEmailDoc f "" s cc scmd content bundle body =
-  sendEmailDoc f cc s "" scmd content bundle body
-sendEmailDoc f t s cc scmd content bundle body =
+             -> Maybe (Doc, Doc) -> Doc -> IO ()
+sendEmailDoc _ "" _ "" _ _ _ = return ()
+sendEmailDoc f "" s cc scmd mbundle body =
+  sendEmailDoc f cc s "" scmd mbundle body
+sendEmailDoc f t s cc scmd mbundle body =
   if have_sendmail || scmd /= "" then do
     withOpenTemp $ \(h,fn) -> do
      hPutDocLn h $
@@ -399,13 +379,17 @@ sendEmailDoc f t s cc scmd content bundle body =
         $$ text ("X-Darcs-Version: " ++ darcs_version)
         $$ body
      hClose h
-     withOpenTemp $ \(hat,at) -> do
-       hPutDocLn hat $ bundle
-       hClose hat
-       let ftable = [('t',t),('c',cc),('f',f),('s',s),
-                     ('b',renderString content), ('a', at)]
-       r <- execSendmail ftable scmd fn
-       when (r /= ExitSuccess) $ fail ("failed to send mail to: " ++ t)
+     ftable' <- case mbundle of
+                Just (content,bundle) ->
+                  withOpenTemp $ \(hat,at) -> do
+                    hPutDocLn hat $ bundle
+                    hClose hat
+                    return [ ('b', renderString content) , ('a', at) ]
+                Nothing ->
+                    return [ ('b', renderString body) ]
+     let ftable = [ ('t',addressOnly t),('c',cc),('f',f),('s',s) ] ++ ftable'
+     r <- execSendmail ftable scmd fn
+     when (r /= ExitSuccess) $ fail ("failed to send mail to: " ++ t)
   else if have_mapi then do
      r <- withCString t $ \tp ->
            withCString f $ \fp ->
@@ -423,6 +407,10 @@ sendEmailDoc f t s cc scmd content bundle body =
   where formated_cc = if cc == ""
                       then empty
                       else text "Cc:" <+> text cc
+        addressOnly a =
+          case dropWhile (/= '<') a of
+          ('<':a2) -> takeWhile (/= '>') a2
+          _        -> a
 
 resendEmail :: String -> String -> PackedString -> IO ()
 resendEmail "" _ _ = return ()
@@ -462,11 +450,11 @@ resendEmail t scmd body =
 execSendmail :: [(Char,String)] -> String -> String -> IO ExitCode
 execSendmail ftable scmd fn =
   if scmd == "" then
-     exec sendmail_path ["-i", "-t"] fn "/dev/null"
+     exec sendmail_path ["-i", "-t"] (File fn, Null, AsIs)
   else case parseCmd (addUrlencoded ftable) scmd of
          Right (arg0:opts, wantstdin) ->
-           do let stdin = if wantstdin then fn else "/dev/null"
-              exec arg0 opts stdin "/dev/null"
+           do let stdin = if wantstdin then File fn else Null
+              exec arg0 opts (stdin, Null, AsIs)
          Left e -> fail $ ("failed to send mail, invalid sendmail-command: "++(show e))
          _ -> fail $ ("failed to send mail, invalid sendmail-command")
 
@@ -497,7 +485,7 @@ execDocPipe c args instr =
       hPutDoc th instr
       hClose th
       withTemp $ \on -> do
-        rval <- exec c args tn on
+        rval <- exec c args (File tn, File on, AsIs)
         case rval of
           ExitSuccess -> readDocBinFile on
           ExitFailure ec -> fail $ "External program '"++c++
@@ -510,7 +498,7 @@ execPipeIgnoreError c args instr =
     withOpenTemp $ \(th,tn) -> do
       hPutDoc th instr
       hClose th
-      withTemp $ \on -> do exec c args tn on
+      withTemp $ \on -> do exec c args (File tn, File on, Stdout)
                            readDocBinFile on
 
 signString :: [DarcsFlag] -> Doc -> IO Doc
@@ -552,7 +540,8 @@ verifyGPG goodkeys s =
       hPutPS th s
       hClose th
       rval <- exec "gpg"  ["--batch","--no-default-keyring",
-                           "--keyring",fix_path goodkeys, "--verify"] tn "/dev/null"
+                           "--keyring",fix_path goodkeys, "--verify"]
+                           (File tn, Null, Null)
       case rval of
           ExitSuccess -> return $ Just gpg_fixed_s
           _ -> return Nothing
@@ -590,7 +579,8 @@ verifySSL goodkeys s = do
                      hClose th
                      writeFilePS cert certdata
                      rval <- exec "openssl" ["smime", "-verify", "-CAfile",
-                                             cert, "-certfile", cert] tn on
+                                             cert, "-certfile", cert]
+                                             (File tn, File on, Null)
                      case rval of
                        ExitSuccess -> Just `liftM` readFilePS on
                        _ -> return Nothing
@@ -615,8 +605,8 @@ getTermNColors :: IO CInt
 
 #ifdef HAVE_CURSES
 
-foreign import ccall "term.h tgetnum" c_tgetnum :: CString -> IO CInt
-foreign import ccall "term.h tgetent" c_tgetent :: Ptr CChar -> CString -> IO CInt
+foreign import ccall "tgetnum" c_tgetnum :: CString -> IO CInt
+foreign import ccall "tgetent" c_tgetent :: Ptr CChar -> CString -> IO CInt
 
 termioBufSize :: Int
 termioBufSize = 4096
@@ -649,6 +639,11 @@ instance Show SSHCmd where
   show SCP  = "scp"
   show SFTP = "sftp"
 
+runSSH :: SSHCmd -> String -> [String] -> [String] -> Redirects -> IO ExitCode
+runSSH cmd remoteAddr preArgs postArgs redirs =
+ do (ssh, args) <- getSSH cmd remoteAddr
+    exec ssh (preArgs ++ args ++ postArgs) redirs
+
 -- | Return the command and arguments needed to run an ssh command
 --   along with any extra features like use of the control master.
 --   See 'getSSHOnly'
@@ -659,11 +654,16 @@ getSSH cmd remoteAddr =
     -- control master
     cmPath <- controlMasterPath remoteAddr
     hasLaunchedCm <- doesFileExist cmPath
-    when (not hasLaunchedCm) $ launchSSHControlMaster remoteAddr
+    when (not hasLaunchedCm && not sshControlMasterDisabled) $
+      launchSSHControlMaster remoteAddr
     hasCmFeature <- doesFileExist cmPath
     let cm_args = if hasCmFeature then [ "-o ControlPath=" ++ cmPath ] else []
+        verbosity = case cmd of
+                    SCP  -> ["-q"] -- (p)scp is the only one that recognises -q
+                                   -- sftp and (p)sftp do not, and plink neither
+                    _    -> []
     --
-    return (ssh, ssh_args ++ cm_args)
+    return (ssh, verbosity ++ ssh_args ++ cm_args)
 
 -- | Return the command and arguments needed to run an ssh command.
 --   First try the appropriate darcs environment variable and SSH_PORT
@@ -701,7 +701,7 @@ hasSSHControlMasterIO = do
   -- the -O flag, but exit with status 255 because of the nonsense
   -- command.  If it does not have the feature, it will simply dump
   -- a help message on the screen and exit with 1.
-  sx <- exec ssh ["-O", "an_invalid_command"] "/dev/null" "/dev/null"
+  sx <- exec ssh ["-O", "an_invalid_command"] (Null,Null,Null)
   case sx of
     ExitFailure 255 -> return True
     _ -> return False
@@ -719,7 +719,7 @@ launchSSHControlMaster rawAddr =
   -- -M : launch as the control master for addr
   -- -N : don't run any commands
   -- -S : use cmPath as the ControlPath.  Equivalent to -oControlPath=
-  exec ssh (ssh_args ++ [addr, "-S", cmPath, "-N", "-f", "-M"]) "/dev/null" "/dev/null"
+  exec ssh (ssh_args ++ [addr, "-S", cmPath, "-N", "-f", "-M"]) (Null,Null,AsIs)
   atexit $ exitSSHControlMaster addr
   return ()
 
@@ -728,7 +728,7 @@ exitSSHControlMaster :: String -> IO ()
 exitSSHControlMaster addr = do
   (ssh, ssh_args) <- getSSHOnly SSH
   cmPath <- controlMasterPath addr
-  exec ssh (ssh_args ++ [addr, "-S", cmPath, "-O", "exit"]) "/dev/null" "/dev/null"
+  exec ssh (ssh_args ++ [addr, "-S", cmPath, "-O", "exit"]) (Null,Null,Null)
   return ()
 
 -- | Create the directory ssh control master path for a given address
