@@ -42,6 +42,7 @@ module FastPackedString (
         readFilePS,  -- :: FilePath -> IO PackedString
         writeFilePS, -- :: FilePath -> PackedString -> IO ()
         gzReadFilePS,-- :: FilePath -> IO PackedString
+        readFileLazily,  -- :: FilePath -> IO LazyFile
         gzReadFileLazily,  -- :: FilePath -> IO LazyFile
         LazyFile(..),
         mmapFilePS,  -- :: FilePath -> IO PackedString
@@ -224,7 +225,7 @@ instance Eq PackedString where
    (==) = eqPS
 
 foreign import ccall unsafe "static string.h memcmp" c_memcmp
-    :: Ptr Word8 -> Ptr Word8 -> Int -> IO Int
+    :: Ptr Word8 -> Ptr Word8 -> CSize -> IO CInt
 
 {-# INLINE eqPS #-}
 eqPS :: PackedString -> PackedString -> Bool
@@ -239,7 +240,8 @@ comparePS (PS _ _ 0) (PS _ _ 0) = EQ    -- short cut for empty strings
 comparePS (PS x1 s1 l1) (PS x2 s2 l2) = unsafePerformIO $ 
     withForeignPtr x1 $ \p1 -> 
         withForeignPtr x2 $ \p2 -> do 
-            i <- c_memcmp (p1 `plusPtr` s1) (p2 `plusPtr` s2) (min l1 l2)
+            i <- c_memcmp (p1 `plusPtr` s1) (p2 `plusPtr` s2)
+                 (fromIntegral $ min l1 l2)
             return $ case i `compare` 0 of
                 EQ  -> l1 `compare` l2
                 x   -> x
@@ -331,14 +333,15 @@ unpackPSfromUTF8 (PS _ _ 0) = ""
 unpackPSfromUTF8 (PS x s l) =
     unsafePerformIO $ withForeignPtr x $ \p->
     do outbuf <- mallocArray l
-       lout <- utf8_to_ints outbuf (p `plusPtr` s) l
+       lout <- fromIntegral `liftM`
+               utf8_to_ints outbuf (p `plusPtr` s) (fromIntegral l)
        when (lout < 0) $ error "Bad UTF8!"
-       str <- (map chr) `liftM` peekArray lout outbuf
+       str <- (map (chr . fromIntegral)) `liftM` peekArray lout outbuf
        free outbuf
        return str
 
 foreign import ccall unsafe "static fpstring.h utf8_to_ints" utf8_to_ints
-    :: Ptr Int -> Ptr Word8 -> Int -> IO Int
+    :: Ptr Int -> Ptr Word8 -> CInt -> IO CInt
 
 -- -----------------------------------------------------------------------------
 -- List-mimicking functions for PackedStrings
@@ -479,22 +482,24 @@ dropWhilePS f ps = seq f $ dropPS (findWhenPS (not . f) ps) ps
 dropWhitePS :: PackedString -> PackedString
 dropWhitePS (PS x s l) =
     unsafePerformIO $ withForeignPtr x $ \p->
-    do i <- first_nonwhite (p `plusPtr` s) l
+    do i <- fromIntegral `liftM`
+            first_nonwhite (p `plusPtr` s) (fromIntegral l)
        return $ if i == l then nilPS
                 else PS x (s+i) (l-i)
 
 foreign import ccall unsafe "fpstring.h first_nonwhite" first_nonwhite
-    :: Ptr Word8 -> Int -> IO Int
+    :: Ptr Word8 -> CInt -> IO CInt
 foreign import ccall unsafe "fpstring.h first_white" first_white
-    :: Ptr Word8 -> Int -> IO Int
+    :: Ptr Word8 -> CInt -> IO CInt
 
 {-# INLINE is_funky #-}
 is_funky :: PackedString -> Bool
-is_funky (PS x s l) = unsafePerformIO $ withForeignPtr x $ \p->
-                      (/=0) `liftM` has_funky_char (p `plusPtr` s) l
+is_funky (PS x s l) =
+    unsafePerformIO $ withForeignPtr x $ \p->
+    (/=0) `liftM` has_funky_char (p `plusPtr` s) (fromIntegral l)
 
 foreign import ccall unsafe "fpstring.h has_funky_char" has_funky_char
-    :: Ptr Word8 -> Int -> IO Int
+    :: Ptr Word8 -> CInt -> IO CInt
 
 
 elemPS :: Char -> PackedString -> Bool
@@ -545,7 +550,7 @@ substrPS tok str
 breakWhitePS :: PackedString -> (PackedString,PackedString)
 breakWhitePS (PS x s l) =
     unsafePerformIO $ withForeignPtr x $ \p->
-    do i <- first_white (p `plusPtr` s) l
+    do i <- fromIntegral `liftM` first_white (p `plusPtr` s) (fromIntegral l)
        if i == 0 then return (nilPS, PS x s l)
                  else if i == l
                       then return (PS x s l, nilPS)
@@ -592,7 +597,7 @@ reversePS :: PackedString -> PackedString
 reversePS ps = packString (reverse (unpackPS ps))
 
 foreign import ccall unsafe "static string.h memcpy" c_memcpy
-    :: Ptr Word8 -> Ptr Word8 -> Int -> IO ()
+    :: Ptr Word8 -> Ptr Word8 -> CSize -> IO ()
 
 concatPS :: [PackedString] -> PackedString
 concatPS [] = nilPS
@@ -608,7 +613,7 @@ concatPS xs
           f ptr len to_go pss@(PS p s l:pss')
            | l <= to_go = do withForeignPtr p $ \pf ->
                                  c_memcpy (ptr `advancePtr` len)
-                                          (pf `advancePtr` s) l
+                                          (pf `advancePtr` s) (fromIntegral l)
                              f ptr (len + l) (to_go - l) pss'
            | otherwise = do let new_total = ((len + to_go) * 2) `max` (len + l)
                             ptr' <- reallocArray ptr new_total
@@ -625,9 +630,10 @@ concatLenPS n [] = n `seq` nilPS
 concatLenPS _ [ps] = ps
 concatLenPS total_length pss = createPS total_length $ \p-> cpPSs p pss
     where cpPSs :: Ptr Word8 -> [PackedString] -> IO ()
-          cpPSs p (PS x s l:rest) = do withForeignPtr x $ \pf ->
-                                          c_memcpy p (pf `plusPtr` s) l
-                                       cpPSs (p `plusPtr` l) rest
+          cpPSs p (PS x s l:rest) =
+              do withForeignPtr x $ \pf ->
+                     c_memcpy p (pf `plusPtr` s) (fromIntegral l)
+                 cpPSs (p `plusPtr` l) rest
           cpPSs _ [] = return ()
 
 {-# INLINE findPS #-}
@@ -638,13 +644,13 @@ findPS c ps = wfindPS (c2w c) ps
 wfindPS :: Word8 -> PackedString -> Maybe Int
 wfindPS c (PS x s l) =
     unsafePerformIO $ withForeignPtr x $ \p->
-    let p' = p `plusPtr` s
-        q = memchr p' (fromIntegral c) (fromIntegral l)
-    in return $ if q == nullPtr then Nothing
+    do let p' = p `plusPtr` s
+       q <- memchr p' (fromIntegral c) (fromIntegral l)
+       return $ if q == nullPtr then Nothing
                                 else Just (q `minusPtr` p')
 
 foreign import ccall unsafe "string.h memchr" memchr
-    :: Ptr Word8 -> CInt -> CSize -> Ptr Word8
+    :: Ptr Word8 -> CInt -> CSize -> IO (Ptr Word8)
 
 {-# INLINE findLastPS #-}
 findLastPS :: Char -> PackedString -> Maybe Int
@@ -790,9 +796,9 @@ foreign import ccall unsafe "static zlib.h gzopen" c_gzopen
 foreign import ccall unsafe "static zlib.h gzclose" c_gzclose
     :: Ptr () -> IO ()
 foreign import ccall unsafe "static zlib.h gzread" c_gzread
-    :: Ptr () -> Ptr Word8 -> Int -> IO Int
+    :: Ptr () -> Ptr Word8 -> CInt -> IO CInt
 foreign import ccall unsafe "static zlib.h gzwrite" c_gzwrite
-    :: Ptr () -> Ptr Word8 -> Int -> IO Int
+    :: Ptr () -> Ptr Word8 -> CInt -> IO CInt
 
 gzReadFilePS :: FilePath -> IO PackedString
 gzReadFilePS f = do
@@ -809,14 +815,17 @@ gzReadFilePS f = do
                  when (gzf == nullPtr) $ fail $ "problem opening file "++f
                  fp <- mallocForeignPtr len
                  debugForeignPtr fp $ "gzReadFilePS "++f
-                 lread <- withForeignPtr fp $ \p -> c_gzread gzf p len
+                 lread <- withForeignPtr fp $ \p ->
+                          c_gzread gzf p (fromIntegral len)
                  c_gzclose gzf
-                 when (lread /= len) $ fail $ "problem gzreading file "++f
+                 when (fromIntegral lread /= len) $
+                      fail $ "problem gzreading file "++f
                  return $ PS fp 0 len
 
 data LazyFile = LazyString String
               | MMappedPackedString PackedString
               | LazyPackedStrings [PackedString]
+    deriving Eq
 
 gzReadFileLazily :: FilePath -> IO LazyFile
 gzReadFileLazily f = do
@@ -832,8 +841,8 @@ gzReadFileLazily f = do
                        fp <- mallocForeignPtr blocksize
                        debugForeignPtr fp $ "gzReadFileLazily "++f
                        lread <- withForeignPtr fp
-                              $ \p -> c_gzread gzf p blocksize
-                       case lread of
+                              $ \p -> c_gzread gzf p (fromIntegral blocksize)
+                       case fromIntegral lread of
                            0 -> do c_gzclose gzf
                                    return []
                            -1 -> fail $ "problem gzreading file "++f
@@ -847,18 +856,32 @@ gzReadFileLazily f = do
             do hClose h
                liftM MMappedPackedString (mmapFilePS f)
 #endif
-        else do let read_rest = do
-                        -- We might be making too big a fp here
-                        fp <- mallocForeignPtr blocksize
-                        debugForeignPtr fp $ "gzReadFileLazily "++f
-                        lread <- withForeignPtr fp
-                               $ \p -> hGetBuf h p blocksize
-                        case lread of
-                            0 -> return []
-                            l -> do rest <- unsafeInterleaveIO read_rest
-                                    return (PS fp 0 l:rest)
-                rest <- unsafeInterleaveIO read_rest
-                return $ LazyPackedStrings (header:rest)
+        else liftM (LazyPackedStrings . (header:)) $ readHandleLazily h
+    where blocksize = 1024
+
+readFileLazily :: FilePath -> IO LazyFile
+readFileLazily f =
+#if defined(__GLASGOW_HASKELL__)
+    if use_mmap
+      then liftM MMappedPackedString (mmapFilePS f)
+      else
+#endif
+           do h <- openBinaryFile f ReadMode
+              liftM LazyPackedStrings $ readHandleLazily h
+
+readHandleLazily :: Handle -> IO [PackedString]
+readHandleLazily h
+ = do let read_rest = do
+              -- We might be making too big a fp here
+              fp <- mallocForeignPtr blocksize
+              debugForeignPtr fp ("readHandleLazily " ++ show h)
+              lread <- withForeignPtr fp
+                     $ \p -> hGetBuf h p blocksize
+              case lread of
+                  0 -> return []
+                  l -> do rest <- unsafeInterleaveIO read_rest
+                          return (PS fp 0 l:rest)
+      unsafeInterleaveIO read_rest
     where blocksize = 1024
 
 hGetLittleEndInt :: Handle -> IO Int
@@ -883,8 +906,9 @@ gzWriteFilePSs f pss  =
 
 gzWriteToGzf :: Ptr () -> PackedString -> IO ()
 gzWriteToGzf gzf (PS x s l) = do
-    lw <- withForeignPtr x $ \p -> c_gzwrite gzf (p `plusPtr` s) l
-    when (lw /= l) $ fail $ "problem in gzWriteToGzf"
+    lw <- withForeignPtr x $ \p -> c_gzwrite gzf (p `plusPtr` s)
+                                                 (fromIntegral l)
+    when (fromIntegral lw /= l) $ fail $ "problem in gzWriteToGzf"
 
 -- -----------------------------------------------------------------------------
 -- mmapFilePS
@@ -908,11 +932,11 @@ mmapFilePS f = if use_mmap
 
 #if defined(__GLASGOW_HASKELL__)
 foreign import ccall unsafe "static fpstring.h my_mmap" my_mmap
-    :: Int -> Int -> IO (Ptr Word8)
+    :: CInt -> CInt -> IO (Ptr Word8)
 foreign import ccall unsafe "static sys/mman.h munmap" c_munmap
-    :: Ptr Word8 -> Int -> IO Int
+    :: Ptr Word8 -> CInt -> IO CInt
 foreign import ccall unsafe "static unistd.h close" c_close
-    :: Int -> IO Int
+    :: CInt -> IO CInt
 #endif
 
 mmap :: FilePath -> IO (ForeignPtr Word8, Int)
@@ -930,7 +954,7 @@ mmap f = do
        else do
 #if defined(__GLASGOW_HASKELL__)
                fd <- fromIntegral `liftM` handleToFd h
-               p <- my_mmap l fd
+               p <- my_mmap (fromIntegral l) fd
                fp <- if p == nullPtr
                      then
 #else
@@ -942,7 +966,9 @@ mmap f = do
                              return thefp
 #if defined(__GLASGOW_HASKELL__)
                      else do
-                             fp <- FC.newForeignPtr p (do {c_munmap p l; return (); })
+                             fp <- FC.newForeignPtr p
+                                   (do {c_munmap p $ fromIntegral l;
+                                        return (); })
                              debugForeignPtr fp $ "mmap "++f
                              return fp
                c_close fd
@@ -963,7 +989,7 @@ mmap f = do
 -- library function strtol.
 
 foreign import ccall unsafe "static stdlib.h strtol" c_strtol
-    :: Ptr Word8 -> Ptr (Ptr Word8) -> Int -> IO CLong
+    :: Ptr Word8 -> Ptr (Ptr Word8) -> CInt -> IO CLong
 
 readIntPS :: PackedString -> Maybe (Int, PackedString)
 readIntPS (PS x s l) =
@@ -979,21 +1005,21 @@ readIntPS (PS x s l) =
 -- fromPS2Hex
 
 foreign import ccall unsafe "static fpstring.h conv_to_hex" conv_to_hex
-    :: Ptr Word8 -> Ptr Word8 -> Int -> IO ()
+    :: Ptr Word8 -> Ptr Word8 -> CInt -> IO ()
 
 fromPS2Hex :: PackedString -> PackedString
 fromPS2Hex (PS x s l) = createPS (2*l) $ \p -> withForeignPtr x $ \f ->
-           conv_to_hex p (f `plusPtr` s) l
+           conv_to_hex p (f `plusPtr` s) $ fromIntegral l
 
 -- -------------------------------------------------------------------------
 -- fromHex2PS
 
 foreign import ccall unsafe "static fpstring.h conv_from_hex" conv_from_hex
-    :: Ptr Word8 -> Ptr Word8 -> Int -> IO ()
+    :: Ptr Word8 -> Ptr Word8 -> CInt -> IO ()
 
 fromHex2PS :: PackedString -> PackedString
 fromHex2PS (PS x s l) = createPS (l `div` 2) $ \p -> withForeignPtr x $ \f ->
-           conv_from_hex p (f `plusPtr` s) (l `div` 2)
+           conv_from_hex p (f `plusPtr` s) (fromIntegral $ l `div` 2)
 
 -- -------------------------------------------------------------------------
 -- betweenLinesPS
