@@ -12,9 +12,6 @@ int pipe( int fildes[2] );
 int renamefile(const char *from, const char *to);
 #endif
 
-int get_raw_mode();
-void set_raw_mode(int raw);
-
 int open_read(const char *fname);
 int open_write(const char *fname);
 int smart_wait(int pid);
