@@ -11,8 +11,9 @@
 -- GNU General Public License for more details.
 --
 -- You should have received a copy of the GNU General Public License
--- along with this program; if not, write to the Free Software Foundation,
--- Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+-- along with this program;  see the file COPYING.  If not, write to
+-- the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+-- Boston, MA 02110-1301, USA.
 
 module Main (main) where
 
@@ -24,22 +25,24 @@ import Data.List ( sort, group )
 import System ( getProgName, getArgs )
 import Monad ( liftM )
 
-intro :: IO String
-intro = do
+intro :: Bool -> IO String
+intro do_stats = do
   prog_name <- getProgName
   return $
     "This is a list of ALL contributors to darcs, sorted according to the\n"++
     "number of patches they have contributed.  This list is automatically\n"++
     "created from the darcs repository, so if there are problems with it,\n"++
     "changes should be made to list_authors.hs.\n"++
-    "\n"++
-    "Run " ++ formatPath (prog_name ++ " stats") ++ " for more detailed information.\n"++
-    "\n"
+    "\n" ++
+    (if do_stats
+     then ""
+     else ("Run " ++ formatPath (prog_name ++ " stats") ++
+           " for more detailed information.\n" ++ "\n"))
 
 main :: IO ()
 main = do darcs_history <- read_repo "."
-          intro >>= putStr
           use_statistics <- elem "stats" `liftM` getArgs
+          intro use_statistics >>= putStr
           mapM_ putStrLn $ sort_authors use_statistics $ map (pi_author.fst)
                          $ concat darcs_history
 
@@ -47,12 +50,13 @@ droundy :: String
 droundy = "David Roundy <droundy@darcs.net>"
 
 -- contributers with more than 2 addresses 
-marnix_klooster, eric_kow, erik_schnetter, mark_stosberg, 
+marnix_klooster, eric_kow, erik_schnetter, edwin_thomson, mark_stosberg,
   thomas_zander, tomasz_zielonka, lele_gaifax :: String
 marnix_klooster = "Marnix Klooster <marnix.klooster@gmail.com>"
 eric_kow        = "Eric Kow <eric.kow@gmail.com>"
 erik_schnetter  = "Erik Schnetter <sschnetter@cct.lsu.edu>"
 mark_stosberg   = "Mark Stosberg <mark@summersault.com>"
+edwin_thomson   = "Edwin Thomson <edwin.thomson@businesswebsoftware.com>"
 thomas_zander   = "Thomas Zander <zander@kde.org>"
 tomasz_zielonka = "Tomasz Zielonka <tomasz.zielonka@gmail.com>"
 lele_gaifax     = "Lele Gaifax <lele@nautilus.homeip.net>"
@@ -69,11 +73,13 @@ authors_sans_name =
   , ("andrew@siaris.net"       , "Andrew L Johnson")
   , ("aoiko@cc.ece.ntua.gr"    , "Aggelos Economopoulos")
   , ("brian@brianweb.net"      , "Brian Alliet")
-  , ("chucky@dtek.chalmers.se" , "Anders Hkersten")
+  , ("chevalier@alum.wellesley.edu", "Kirsten Chevalier")
+  , ("chucky@dtek.chalmers.se" , "Anders Hockersten")
   , ("dbindner@truman.edu"     , "Don Bindner")
   , ("dons@cse.unsw.edu.au"    , "Don Stewart")
   , ("egli@apache.org"         , "Christian Egli")
   , ("ei@vuokko.info"          , "Esa Ilari Vuokko")
+  , ("era+darcs@iki.fi"        , "Era Eriksson")
   , ("eric@rti-zone.org"       , "Eric Gaudet")
   , ("fw@deneb.enyo.de"        , "Florian Weimer")
   , ("glaweh@physik.fu-berlin.de", "Henning Glawe")
@@ -87,6 +93,7 @@ authors_sans_name =
   , ("kannan@cakoose.com"      , "Kannan Goundan")
   , ("ketil@ii.uib.no"         , "Ketil Malde")
   , ("lord@crocodile.org"      , "Vadim Zaliva")
+  , ("malebria@riseup.net"     , "Marco Tulio Gontijo e Silva")
   , ("me@JonathonMah.com"      , "Jonathon Mah")
   , ("naesten@myrealbox.com"   , "Samuel Bronson")
   , ("nils@ndecker.de"         , "Nils Decker")
@@ -125,6 +132,9 @@ canonize_author "kow@loria.fr" = eric_kow
 canonize_author "Eric Kow <eric.kow@loria.fr>" = eric_kow
 canonize_author "schnetter@aei.mpg.de" = erik_schnetter
 canonize_author "Erik Schnetter <schnetter@aei.mpg.de>" = erik_schnetter
+canonize_author "edwin.thomson@businesswebsoftware.com" = edwin_thomson
+canonize_author "edwin.thomson@gmail.com" = edwin_thomson
+canonize_author "Kirill Smelkov <kirr@mns.spb.ru>" = "Kirill Smelkov <kirr@landau.phys.spbu.ru>"
 canonize_author "mark@summersault.com" = mark_stosberg
 canonize_author "jvr+darcs@blub.net" = "Joeri van Ruth <jvr@blub.net>"
 canonize_author "testerALL --ignore-times" = mark_stosberg
