@@ -11,8 +11,9 @@
 -- GNU General Public License for more details.
 --
 -- You should have received a copy of the GNU General Public License
--- along with this program; if not, write to the Free Software Foundation,
--- Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+-- along with this program;  see the file COPYING.  If not, write to
+-- the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+-- Boston, MA 02110-1301, USA.
 
 module Main (main) where
 
@@ -164,7 +165,7 @@ entryFile = do es <- many entry
 
 loadEntryFile :: FilePath -> IO [ChangeLogEntry]
 loadEntryFile fname = do
-    cs <- readFile fname
+    cs <- readFile fname -- ratify readFile: not part of darcs executable
     case parse entryFile fname cs of
         Left err -> fail (show err)
         Right x -> return x
