@@ -65,3 +65,25 @@ cd temp3
 $DARCS check
 cd ..
 rm -rf temp1 temp2 temp3
+
+# Check that pending looks ok
+mkdir temp1
+cd temp1
+$DARCS init
+mkdir d
+$DARCS add d
+$DARCS rec -Ax -am 'add d'
+$DARCS tag -Ax t
+rmdir d
+$DARCS rec -Ax -am 'rm d'
+cd ..
+$DARCS get --tag t temp1 temp2
+cd temp2
+if [ -f _darcs/patches/pending ]; then
+	if grep -qv '^[{}]$' _darcs/patches/pending; then
+		cat _darcs/patches/pending
+		exit 1
+	fi
+fi
+cd ..
+rm -rf temp1 temp2
