@@ -14,7 +14,7 @@ echo b > foo
 echo y | $DARCS revert -a
 echo y/d/y | tr / \\012 | $DARCS unrecord
 # since the unrevert is impossible, we should fail if it succeeds...
-! echo y | $DARCS unrevert
+! echo y | $DARCS unrevert || exit 1
 
 # now let's try a possible unrevert, just for fun...
 echo b >> foo
