@@ -23,28 +23,28 @@ $DARCS mv fun fum ..
 $DARCS record --author me --all --no-test --patch-name mv
 cd ..
 
-test ! -f _darcs/current/fee
+test ! -f _darcs/pristine/fee
 test ! -f fee
-cmp d/fee _darcs/current/d/fee
+cmp d/fee _darcs/pristine/d/fee
 
 test ! -f fi
 test -f d/fi
-test ! -f _darcs/current/fi
-test ! -f _darcs/current/d/fi
+test ! -f _darcs/pristine/fi
+test ! -f _darcs/pristine/d/fi
 
 test ! -f fo
 test -f d/fo
-test ! -f _darcs/current/fo
-test ! -f _darcs/current/d/fo
+test ! -f _darcs/pristine/fo
+test ! -f _darcs/pristine/d/fo
 
-cmp fun _darcs/current/fun
-cmp fum _darcs/current/fum
+cmp fun _darcs/pristine/fun
+cmp fum _darcs/pristine/fum
 
 $DARCS mv fun d
 $DARCS record -A me -a --no-test -m "fun again"
-cmp d/fun _darcs/current/d/fun
+cmp d/fun _darcs/pristine/d/fun
 test ! -f fun
-test ! -f _darcs/current/fun
+test ! -f _darcs/pristine/fun
 
 # Now clean up.
 cd ..
