@@ -21,7 +21,7 @@ cd ..
 $DARCS get --repo-name temp2 temp1
 cd temp2
 cp ../temp1/_darcs/prefs/defaults _darcs/prefs
-echo y | $DARCS unpull
+echo y/d/y | tr / \\012 | $DARCS unpull --interactive
 test -e foo -a ! -s foo
 chmod +x foo
 test -x foo
