@@ -49,10 +49,7 @@ echo hello world > bar
 $DARCS add bar
 $DARCS replace hello goodbye bar
 
-$DARCS revert <<EOF
-nnyy
-y
-EOF
+echo "nnyy/y" | tr / \\012 | $DARCS revert
 
 DARCS_DONT_COLOR=1 $DARCS wh > whatsnew
 cat > correct <<EOF
