@@ -17,6 +17,11 @@ $DARCS whatsnew
 $DARCS whatsnew --summary
 echo y | $DARCS revert -a
 cmp foo _darcs/pristine/foo
+mkdir d
+$DARCS add d
+$DARCS record -a -m 'add dir' -A x
+rmdir d
+$DARCS revert -a d
 cd ..
 rm -rf temp1
 
