@@ -23,12 +23,12 @@ darcs "add b.txt";
 darcs "record -A x -a -m 'adding b' b.txt";
 
 like(
-  echo_to_darcs("a\n","unpull -p add"),
+  echo_to_darcs("an","unpull -p add"),
   qr/really unpull/i, 
   "additional confirmation is given when 'all' option is selected");
 
 like(
-  echo_to_darcs("n\n","unpull --last 1"),
+  echo_to_darcs("n","unpull --last 1"),
   qr/adding b/, 
   "unpull --last 1 gives expected result");
 
