@@ -57,7 +57,7 @@ if ($^O =~ /msys/i) {
     close(FOO);
     touch('date.t');
     darcs(qw/add date.t/);
-    like( darcs('whatsnew', $abs_repo_path."date.t"), qr/hunk/i, $test_name);
+    like( darcs('whatsnew', $abs_repo_path."/date.t"), qr/hunk/i, $test_name);
 }
 
 chdir '../';
